from Node import *
import os,re,string

# transfer bifurcation at root to trifurcation
def __unroot(root,get_bl): 
    c=1 #prefer right branch to delete (only take left if right is a leaf
    if len(root.children[c].children)==0:c=0
    k=1-c #children to keep
    if get_bl:root.children[k].branch+=root.children[c].branch #transfer branch length
    child=root.children[c].children
    for ch in child:
        ch.ancestor=root
    root.children=[root.children[k]]+child


# Parse a newick string and create a node representation
# unroot changes a tree which has a bifurcation at the root into one with a trifurcation
def parse_newick(newick_string, get_bl=True,get_int=False,unroot=False): 
    root=Node(None)
    root.branch=0
    root.name='root'
    ancnode=None
    actnode=root
    bl,name=False,False
    i=1
    for char in newick_string:
        if bl:
            if char==')' or char==',':
                actnode.branch=float(branch)
                bl=False
            else:
                branch+=char
                continue
        elif name:
            if char==')' or char==',' or char=='(' or char==":":
                if len(namestr)==0:
                    namestr="inner_%u"%i
                    i+=1
                actnode.name=namestr
                name=False
            else:
                namestr+=char
                continue
            
        if char=='(':ancnode=actnode
        if char=='(' or char==',':
            actnode=Node(ancnode) #New node
            ancnode.set_child(actnode)
            namestr=""
            name=True
        elif char==')':
            actnode=ancnode
            ancnode=actnode.ancestor
            if get_int:
                namestr=""
                name=True
        elif get_bl and char==':':
            bl=True
            branch=""
    if unroot and len(root.children)==2:
        __unroot(root,get_bl)
    return root

# reroot the node representation at a particular node
# also for multifurcating trees
# if support, bl are interpreted as support values, same unroot as in reading tree
def __reroot_at_node(node,support=False,unroot=False): 
    if not node.ancestor:
        if unroot and len(node.children)==2:
            __unroot(node,True) 
        return node
    
    oldanc=node.ancestor
    if not oldanc.ancestor: # already at root, only need to unroot
        node=oldanc
        if unroot and len(oldanc.children)==2:
            __unroot(node,True)
        return node
    node.ancestor=Node(None) #create new root node
    root=node.ancestor
    root.branch=0
    root.name='root'
    keeproot=root
    if not support:
        node.branch=node.branch/2.0
        oldbr1=node.branch/2.0
    else: oldbr1=node.branch
    root.children=[node,oldanc]
    root=node
    while oldanc.name!='root':
        oldbr2=oldanc.branch  #change branch lengths
        oldanc.branch=oldbr1
        oldbr1=oldbr2
        newanc=oldanc.ancestor       #change ancestor and children
        for i in range(0,len(oldanc.children)):
            if oldanc.children[i]==root:
                oldanc.children[i]=newanc
                break
        oldanc.ancestor=root
        root=oldanc    #repeat with next node till root found
        oldanc=newanc
        
    if len(oldanc.children)==2: #delete node cause only one child then
        
        if oldanc.children[0]==root:sister=oldanc.children[1]  #delete old root and change pointer there
        else:sister=oldanc.children[0]
        if not support:sister.branch+=oldbr1
        sister.ancestor=root
        for i in range(0,len(root.children)):
            if root.children[i]==oldanc:root.children[i]=sister
        del oldanc
    
    else: #only change pointers, oldroot becomes new inner node with 1 children less
        for i in range(0,len(oldanc.children)):
            if oldanc.children[i]==root:
                newchil=oldanc.children[:i]+oldanc.children[i+1:]
                break
        oldanc.children=newchil
        oldanc.ancestor=root
        oldanc.branch=oldbr1
        oldanc.name='oldroot'

    if unroot and len(keeproot.children)==2:
        __unroot(keeproot,True)
    return keeproot

# return the node corresponding to the future root
def __reroot_with_taxa(noderep,root): 
    if type(root)==str:root=root.split() #else already list
    rootnode=noderep
    all_leaves=set(rootnode.get_leaves())
    
    outgroup=set(root) #set of taxa name
    outgroup2=all_leaves.difference(outgroup)        

    for node in rootnode.traverse():
        if not node.ancestor:continue #root node
        s=set(node.get_leaves())
        if not node.ancestor.ancestor: #node attached to root, only consider outgroup
            if s==outgroup:
                return node
        elif s==outgroup or s==outgroup2: #either in one of the taxa sets defined by edges to children or edge to ancestor
            return node
    # not found 
    print "ERROR:",root,"is not contained in tree or not monophyletic"
    return None


# reroot the node representation at a particular taxon
# noderep is old node representation, root may be string of several taxa delimited by spaces
def reroot(noderep,root,support=False,unroot=False): 
    node=__reroot_with_taxa(noderep,root,)
    rootnode=__reroot_at_node(node,support,unroot=unroot)
    for node in rootnode.traverse():
        node.leaves=[] #del leaves bec structure changed
        node.depth=node.get_depth() #set new depth
    return rootnode

# Create subtree by deleting the taxa deltaxa, return as node or newick
def subtree(tree,deltaxa,asnode=False):
    def delete_taxa(root,deltaxa): #root Node and taxa list
        for node in root.traverse():
            if node.name in deltaxa and len(node.children)==0:
                while True: #maybe repeatly delete nodes bec. inner node becomes leaf (occures rarely)
                    c=node.ancestor
                    if c!=None:
                        if len(c.children)>1:
                            i=c.children.index(node)
                            c.children=c.children[0:i]+c.children[i+1:]
                            del node
                            break
                        else:
                            del node
                            node=c
        return root

    # collapse Nodes with only one child and sum up branch lengths
    def collapse_two(root): 
        while True: #set root to first node with >1 children
            if len(root.children)>1:break
            if len(root.children)==0:return None
            root=root.children[0]
            root.name='root'
            root.ancestor=None
            
        for node in root.traverse():
            if len(node.children)==1:
                c=node.children[0]
                a=node.ancestor
                c.branch+=node.branch
                c.ancestor=a
                if a!=None:
                    i=a.children.index(node)
                    a.children=a.children[:i]+[c]+a.children[i+1:]
                del node
        return root

    if asnode:root=tree
    else: root=parse_newick(tree)
    root=delete_taxa(root,deltaxa) #manipulates root and children
    root=collapse_two(root)
    if asnode:return root
    else: return root.newick()

# return first tree, newick or nexus file
def read_tree(input):
    infile=open(input)
    fline=infile.readline()
    if re.compile("#nexus", re.IGNORECASE).search(fline)!=None:format='nexus'
    else: format='phylip'
    if format=='nexus':
        trees=False
        pstart=re.compile("^Begin trees;", re.IGNORECASE)
        ptree=re.compile("tree.*=.*?(\(.*\));", re.IGNORECASE)
        for line in infile.readlines():
            if not trees:
                if pstart.search(line) != None:trees=True
                continue
            ftree=ptree.search(line)
            if ftree == None: continue
            return ftree.groups()[0]
    elif format=='phylip':    #like fitch output
        for line in infile.readlines():
            fline+=line
        fline=fline.replace('\n','')
        return fline[:fline.find(';')] 

# read treelist out of newick file
def get_treelist(file): 
    inf=open(file).readlines()
    if re.compile("#nexus", re.IGNORECASE).search(inf[0])!=None:return get_treelist_nexus(file)
    return [i[:-2] for i in inf]

# read treelist out of nexus file
def get_treelist_nexus(file):
    trees=False
    treelist=[]
    pstart=re.compile("^Begin trees;", re.IGNORECASE)
    ptree=re.compile("tree.*=.*?(\(.*\));", re.IGNORECASE)
    pend=re.compile("end;", re.IGNORECASE)
    for line in open(file).readlines():
        if not trees:
            if pstart.search(line) != None:trees=True
            continue
        if pend.search(line)!= None:return treelist
        ftree=ptree.search(line)
        if ftree == None: continue
        treelist.append(ftree.groups()[0])
        
# write treelist to phylip file
def write_treelist_phylip(outname,trees):
    open(outname,'w').write(string.join(trees,";\n")+";\n")

# compute mean
def mean(l):
    return(sum(l)/float(len(l)))

####### Methods for bitvectors ###########
# count the number of 1s in an int
def bitcount(x):
    n = 0
    while x:
        x = x & (x-1)
        n = n + 1
    return n

# get index of k-th bit that is one, dont check for overflow!
# the first bit is k==1, not 0 as in lists
def bitnumber(x,k): 
    i=0 #counts position in sequence
    j=0 #counts number of ones
    while True:
        if x&(1<<i):j+=1
        if j==k:return i
        i+=1

