#! /usr/bin/env python

from Node import *
from copy import *
import random,myutils

# insert new node with name name as sister to node
def insert_node(node,name=""): 
    newinnernode=Node(node.ancestor)
    newouternode=Node(newinnernode)  
    newinnernode.children=[newouternode,node]
    i=node.ancestor.children.index(node)
    node.ancestor.children=node.ancestor.children[:i]+node.ancestor.children[i+1:]+[newinnernode]
    node.ancestor=newinnernode
    newouternode.name=name

# return random tree under the PDA model
def random_equ_tree(taxa,newick=True,rand=False): # if rand, taxa randomized, only needed for rooted trees 
    if rand:
        newtaxa=deepcopy(taxa)
        random.shuffle(newtaxa)
    else:newtaxa=taxa    
    root=Node(None)
    l1=Node(root)
    l1.name=newtaxa[0]
    l2=Node(root)
    l2.name=newtaxa[1]
    root.children=[l1,l2]
    for j in range(2,len(newtaxa)):
        # can insert above each node, except above root and above left son of root, since unrooted trees
        nodes=root.traverse() 
        node=root
        while (node==root or node==root.children[0]): node=random.choice(nodes)
        insert_node(node,newtaxa[j])
    if not newick: return root
    return root.newick()

# return random tree
def random_tree(taxa,rand=False): 
    return random_equ_tree(taxa,rand=rand)

def list_join(l): #joins lists of lists 
    return [i for j in l for i in j]

# insert the taxa at all possible positions and return new nodes
def all_trees(node,taxa): 
    t=taxa[0]
    nodes=[]
    for n in node.traverse():
        if n.name=="root":continue
        newnode=node.copy()
        newnode.insert_taxon(n.name,t)
        nodes.append(newnode)
    if len(taxa)==1:return [nodes] #nothing to add
    return list_join([all_trees(n,taxa[1:]) for n in nodes])


# generate all rooted trees for the set of taxa
def all_rooted_trees(taxa):  
    startnode=myutils.parse_newick("(%s,%s,rt)" %(taxa[0],taxa[1])) #rt is root taxa
    all=list_join(all_trees(startnode,taxa[2:]))
    newn=[]
    for node in all:
        newnode=myutils.reroot(node,"rt",unroot=True)
        c=newnode.children
        rtnode=[i for i in range(0,3) if c[i].name=="rt"][0]
        newnode.children=newnode.children[0:i]+newnode.children[i+1:]
        newn.append(newnode)
    return newn       

# resove all multifurcations, but no multifurcations > cutoff (without cutoff there may be too many trees to generate and memory problems, good cutoff is 7)
def res_all_mult(node,cutoff=0): 
    nlist=node.traverse()
    for i in range(0,len(nlist)):
        n=nlist[i]
        if len(n.children)>3 or (len(n.children)==3  and not n.name=="root") : #so with unrooted trees
            if (cutoff and len(n.children)>=cutoff):return None 
            taxa=["d"+str(i) for i in range(0,len(n.children))]
            # generate subtree of that size and insert into multifurcation,
            # randomizing names neccessary cause subtrees are rooted!
            newtrees=all_rooted_trees(taxa) 
            newnodes=[]
            for ntree in newtrees:
                newnode=node.copy()
                newn=[x for x in newnode.traverse() if x.name==n.name][0]
                oldchil=newn.children
                newn.children=ntree.children
                for c in newn.children:
                    c.ancestor=newn
                l=ntree.get_leaves(names=False)
                l.sort(key=lambda n:n.name) # sort by names, makes sure that really random rooted subtrees
                for i in range(0,len(l)):
                    a=l[i].ancestor
                    j=a.children.index(l[i])
                    a.children=a.children[0:j]+[oldchil[i]]+a.children[j+1:]
                    oldchil[i].ancestor=a
                newnodes.append(newnode)
            # recursion starts
            # multifurcation, all have to be considered independently
            res=[res_all_mult(node,cutoff) for node in newnodes] 
            if None in res:return None
            return list_join(res)
    return [node] #no multifurcation


